import Taro from '@tarojs/taro';

import { Dialog } from '@taroify/core';
import { useCallback, useEffect, useRef, useState } from 'react';

import { PrejobPopup } from '@/components/prejob-popup';
import { DialogStatus, PREFIX } from '@/components/product-dialog/const';
import JobBuy from '@/components/product-dialog/steps-ui/job-buy';
import ContactCustomerService from '@/components/product-dialog/steps-ui/job-contact-customer';
import ContactDirect from '@/components/product-dialog/steps-ui/job-contact-direct';
import UnableUnlockContent from '@/components/product-dialog/steps-ui/job-unable';
import { EventName } from '@/constants/app';
import { GET_CONTACT_TYPE } from '@/constants/job';
import { DeclarationType, ProductType } from '@/constants/product';
import { JobDetails } from '@/types/job';
import { ProductInfo } from '@/types/product';
import { logWithPrefix } from '@/utils/common';
import {
  requestAllBuyProduct,
  requestProductBalance,
  requestProductUseRecord,
  requestUseProduct,
} from '@/utils/product';
import Toast from '@/utils/toast';

import '../index.less';

interface IProps {
  data: JobDetails;
  onClose: () => void;
}

const PRODUCT_CODE = ProductType.VIP;
const log = logWithPrefix('product-job-dialog');

function ProductJobDialog(props: Omit<IProps, 'visible'>) {
  const { data, onClose } = props;
  const [status, setStatus] = useState<DialogStatus>(DialogStatus.LOADING);
  const [showContact, setShowContact] = useState(false);
  const [showPrejob, setShowPrejob] = useState(false);
  const [publisherAcctNo, setPublisherAcctNo] = useState<string>('');
  const initRef = useRef<(skipPreAction?: boolean, skipRecordGet?: boolean) => void | Promise<void>>(() => {});

  const handleCloseDialog = useCallback(() => {
    setShowContact(false);
    onClose();
  }, [onClose]);

  const handleClosePrejob = useCallback(() => {
    setShowPrejob(false);
    onClose();
  }, []);

  const handleConfirmPrejob = useCallback(async (type: GET_CONTACT_TYPE) => {
    if (type === GET_CONTACT_TYPE.VIP) {
      const allowBuy = await requestAllBuyProduct(PRODUCT_CODE);
      setShowContact(true);
      setShowPrejob(false);
      setStatus(allowBuy ? DialogStatus.PREACTION_JOB_BUY : DialogStatus.JOB_UNABLE_UNLOCK);
    } else if (type === GET_CONTACT_TYPE.VIDEO) {
      Taro.eventCenter.trigger(EventName.SHOW_VIDEO_REWARD);
    }
  }, []);

  const handleAfterBuy = useCallback(async () => {
    const [time] = await requestProductBalance(PRODUCT_CODE);
    if (time <= 0) {
      Toast.error('发生错误请重试');
      onClose();
      return;
    }
    const productInfo = await requestUseProduct(PRODUCT_CODE, { jobId: data.id });
    const declarationTypeResult = productInfo.declarationTypeResult;
    setShowContact(true);
    setShowPrejob(false);
    if (declarationTypeResult?.type === DeclarationType.Direct && declarationTypeResult.publisherAcctNo) {
      setStatus(DialogStatus.JOB_CONTACT_DIRECT);
      setPublisherAcctNo(declarationTypeResult.publisherAcctNo);
    } else {
      setStatus(DialogStatus.JOB_CONTACT_CS);
    }
  }, [data, onClose]);

  const handleReport = useCallback(() => {
    log('report', data.id);
  }, [data]);

  useEffect(() => {
    initRef.current = async (skipPreAction?: boolean, skipRecordGet?: boolean) => {
      const handleContact = (declarationTypeResult?: ProductInfo['declarationTypeResult']) => {
        setShowContact(true);
        if (declarationTypeResult?.type === DeclarationType.Direct && declarationTypeResult.publisherAcctNo) {
          setStatus(DialogStatus.JOB_CONTACT_DIRECT);
          setPublisherAcctNo(declarationTypeResult.publisherAcctNo);
        } else {
          setStatus(DialogStatus.JOB_CONTACT_CS);
        }
      };
      try {
        Taro.showLoading();
        // if (1 < 2) {
        //   setStatus(DialogStatus.JOB_CONTACT_CS);
        //   return;
        // }
        if (!skipRecordGet) {
          const result = await requestProductUseRecord(PRODUCT_CODE, { jobId: data.id });
          log('requestProductUseRecord result', result);
          if (result) {
            handleContact(result.declarationTypeResult);
            return;
          }
        }
        const [time, isPaidVip] = await requestProductBalance(PRODUCT_CODE);
        if (time <= 0) {
          // const allowBuy = await requestAllBuyProduct(PRODUCT_CODE);
          setShowContact(true);
          setStatus(DialogStatus.JOB_UNABLE_UNLOCK);
        } else if (isPaidVip || skipPreAction) {
          const productInfo = await requestUseProduct(PRODUCT_CODE, { jobId: data.id });
          setShowPrejob(false);
          handleContact(productInfo.declarationTypeResult);
        } else {
          setShowPrejob(true);
          setStatus(DialogStatus.PRE_ACTION);

          // const productInfo = await requestUseProduct(PRODUCT_CODE, { jobId: data.id });
          // handleContact(productInfo.declarationTypeResult);
        }
      } catch (e) {
        console.log(e);
        Toast.error('出错了，请重试');
        handleCloseDialog();
      } finally {
        Taro.hideLoading();
      }
    };
  }, [data, handleCloseDialog]);

  useEffect(() => {
    initRef.current();
  }, []);

  useEffect(() => {
    Taro.eventCenter.once(EventName.PRE_ACTION_SHARE_SUCCESS, () => {
      initRef.current(true, true);
    });
    Taro.eventCenter.on(EventName.PRE_ACTION_VIDEO_SUCCESS, () => {
      initRef.current(true, true);
    });

    return () => {
      Taro.eventCenter.off(EventName.PRE_ACTION_SHARE_SUCCESS);
      Taro.eventCenter.off(EventName.PRE_ACTION_VIDEO_SUCCESS);
    };
  }, []);

  if (status === DialogStatus.LOADING) {
    return null;
  }

  return (
    <div>
      <PrejobPopup open={showPrejob} onCancel={handleClosePrejob} onConfirm={handleConfirmPrejob} />
      <Dialog className={PREFIX} onClose={onClose} open={showContact}>
        <Dialog.Content>
          {status === DialogStatus.JOB_CONTACT_CS && <ContactCustomerService onAfterConfirm={handleCloseDialog} />}
          {status === DialogStatus.JOB_CONTACT_DIRECT && (
            <ContactDirect
              publisherAcctNo={publisherAcctNo}
              onAfterConfirm={handleCloseDialog}
              onReport={handleReport}
            />
          )}
          {(status === DialogStatus.JOB_BUY || status === DialogStatus.PREACTION_JOB_BUY) && (
            <JobBuy onConfirm={handleAfterBuy} justBuy={status === DialogStatus.PREACTION_JOB_BUY} />
          )}
          {status === DialogStatus.JOB_UNABLE_UNLOCK && <UnableUnlockContent onConfirm={handleCloseDialog} />}
        </Dialog.Content>
      </Dialog>
    </div>
  );
}

export default ProductJobDialog;
